
##########################
##########################
# General procedures.
##########################
##########################




factorcoeffspol:=proc(f,x,n) # Gives pol. f(x) upto degree n with factorised
                             # coefficients.
local ff,i,coefff,newcf,j:
ff:=collect(f,x);
for i from 0 to n do
  coefff[i]:=coeff(ff,x,i);
  newcf(i):=factor(coefff[i]);
od;
sum('(newcf(j))*x^j',j=0..n);
end:



##########################



integercoeffpol:=proc(f) # Multiplies a pol. f with the lcm of the denominators
                         # of the rational parts of the coefficients.
                         # It gives a function with integer coefficients
                         # besides the occurring unknowns.
local ff,coeffen,nopscoeff,lcmdenom,i;
ff:=expand(f); 
coeffen:={coeffs(ff)};
nopscoeff:=nops(coeffen);
lcmdenom:=1;
for i from 1 to nopscoeff do
  lcmdenom:=lcm(lcmdenom,denom(simplify(coeffen[i])));
od;
simplify(lcmdenom*ff);
end:



##########################



convertpolintsquarefree:=proc(ar)  # converts the array of factors(polynom)
                                 # into a squarefree polynomial by taking the 
                                 # parts of degree >0 such that they have
                                 # integer coefficients. 
                                 # The ordering of the factors is executed
                                 # as given in the array.
                                 # The received polynomial however might
                                 # have an other ordering.
local polpartar,nopsar,pol,i,polpartari;
polpartar:=ar[2]; 
nopsar:=nops(polpartar);
pol:=1;
for i from 1 to nopsar do
  polpartari:=integercoeffpol(polpartar[i][1]);
  pol:=pol*polpartari;
od;
if ar[1]=0 then
  pol:=0;
fi;
pol;
end:



##########################



reducebivarpols:=proc(originallisteqns,var1,var2,A,B) 
  # We construct a minimal list of arrays [f_A,f_B,f_B/A]
  # for the variables var1, var2.
  # We have f_A in Q[A], f_B in Q[B] and f_B/A in Q[A,B].
  # They are irreducible (if ``factor'' works in the way it should).
  # Such a polynomial P(A,B) in the array satisfies P(var1,var2)=0.


local listsolutions,numbeqns,newlisteqns,i,eqn,setconstants,setA,setB,\  
        listAB,nopsAB,polconstants,polA,polB,polAB,gcdAB,listgcd,\
        reducedlistAB, newpol,newpolA,factorspolA,\
        numbfactpolA,k,factpolA, alpha,listalphaB,substeqn,\
        eqnBalpha,factorspolAB,numbfactpolAB,m,factpolAB,factorspolB,\
        s,eqnBrational,factpolB,listeqns:

# Make the equations squarefree with integral coefficients.
# We obtain the new list newlisteqns.


listeqns:=originallisteqns;
listsolutions:=[]; 

numbeqns:=nops(listeqns);
newlisteqns:=[];
for i from 1 to numbeqns do  
  eqn:=expand(convertpolintsquarefree(factors(listeqns[i]))); 
newlisteqns:=[op(newlisteqns),eqn]; 
od;
eqn:='eqn';


# Seperate the equations in  four sets of, constants, eqns in A,
# equations in B and equations in AB.
# Put them in sets.

setconstants:={}; 
setA:={};
setB:={};
listAB:=[];
for i from 1 to numbeqns do  
  if degree(newlisteqns[i],A)=0 and degree(newlisteqns[i],B)=0 then
    setconstants:=setconstants union {newlisteqns[i]};
  elif degree(newlisteqns[i],A)>0 and degree(newlisteqns[i],B)=0 then
    setA:=setA union {newlisteqns[i]};
  elif degree(newlisteqns[i],A)=0 and degree(newlisteqns[i],B)>0 then
    setB:=setB union {newlisteqns[i]};
  else
    listAB:=[op(listAB),newlisteqns[i]];
  fi;
od;
nopsAB:=nops(listAB);
 
# print(setconstants);
# print(setA);
# print(setB);
# print(listAB);
# print(nopsAB);


# Make gcds out of two equations of the sets of constants, A and B:

polconstants:=0;
polA:=0;
polB:=0;
for i from 1 to nops(setconstants) do
  polconstants:=gcd(polconstants,setconstants[i]);
od;
for i from 1 to nops(setA) do
  polA:=gcd(polA,setA[i]);
od;
for i from 1 to nops(setB) do
  polB:=gcd(polB,setB[i]);
od;
polA:=integercoeffpol(polA); 
polB:=integercoeffpol(polB);

# Check for cases that does not give solutions for A and B or
# that only gives one equation or independent equations:

if polconstants<>0 then
  lprint(`  --There are NO solutions for `, var1,` and `, var2);  
  # lprint();
  RETURN([[]]);
elif degree(polA,A)=0 and polA<>0 then
  lprint(`  --There are NO solutions for `, var1,` and `, var2);
  # lprint();
  RETURN([[]]);
elif degree(polB,B)=0 and polB<>0 then
  lprint(`  --There are NO solutions for `, var1,` and `, var2);
  # lprint();
  RETURN([[]]);
elif nopsAB=0 then
  if polA<>0 then
    listsolutions:=[[polA,polB,0]];
  else
    listsolutions:=[[polA,polB,polB]];
  fi;
  lprint(`  --The polynomials for `, var1,` and `,var2,` are independent and lead to the solution`);
  print(listsolutions);
  RETURN(listsolutions);
elif polA=0 and polB=0 and nopsAB=1 then
  listsolutions:=[[0,0,listAB[1]]];
  lprint(`  --There is ONE nontrivial relation leading to the solution`);
  print(listsolutions);
  # lprint();
  RETURN(listsolutions);
fi;

# We now have listAB is non-empty, polA and polB are zero if they are constant.
# Now the computing part begins.
# First we make a new equation in A and B with the help of gcd's
# and the sets.

polAB:=0;
for i from 1 to nopsAB do
  polAB:=gcd(polAB,listAB[i]); 
od;

# Notice that polAB is non-zero, since nopsAB=0 was already treated.

gcdAB:=0;
gcdAB:=gcd(gcdAB, polA);
gcdAB:=gcd(gcdAB, polB);
gcdAB:=gcd(gcdAB, polAB); 

# This gcdAB is non-zero, since polAB is.
# So we have a gcdAB of degree at least 0 in one of te variables A and B.

if type(gcdAB,complex)=false then
  print(`--From the list we made a non-constant gcd of the entries: `, gcdAB);
  lprint(`  --This gcd gives rise to TWO new lists of polynomials`);
  lprint(`    that should give all solutions to our problem.`);
  lprint(`    First make a GCD list. It consist of this gcd.`);
  lprint(`    The entries, divided by the gcd, is put in the REDUCED list.`);
  lprint(`    The REDUCED list can contain one or two extra 0's.`);
  # lprint();
    
  listgcd:=[gcdAB];
     
  polA:=simplify(polA/gcdAB);
  polB:=simplify(polB/gcdAB);
  reducedlistAB:=[];
  for i from 1 to nopsAB do
    newpol:=simplify(listAB[i]/gcdAB);
    reducedlistAB:=[op(reducedlistAB),newpol];
  od; 
  newpol:='newpol';

# We have made two new lists "listgcd" and "reducedlistAB".
# polA and polB are replaced by polA/gcdAB and polB/gcdAB, respectively.
# We shall rename reducedlistAB as listAB. It has nopsAB entries.

  print(`--Calculations for the new list.`);
  lprint(`    The GCD-list w.r.t. the gcd is:`,listgcd);
  # lprint();
  listsolutions:=reducebivarpols(listgcd,var1,var2,A,B);
 
  print(`--We continue with the reduced list.`);
  lprint(`    The REDUCED list of polynomials is given by:`, [polA, polB,  \ 
             op(reducedlistAB)]);
  # lprint();
  listAB:=reducedlistAB;
  nopsAB:=nops(listAB);
  reducedlistAB:='reducedlistAB';
fi;

# So we now have polA, polB, polAB and listAB that have a common constant gcd.
# We are going to make a non-constant polynomial for A.

if polA=0 and polB=0 then 
  i:=2;
  while polA=0 do
    polA:=resultant(listAB[1],listAB[i],B);  # For at least one i this is 
                                             # non-zero. Since otherwise
                                             # we would have deg(gcd)>0.
    polA:=convertpolintsquarefree(factors(polA)); 
    i:=i+1;
  od;
  i:='i';
  if nopsAB>=4 then
    newpolA:=resultant(listAB[3],listAB[4],B);
    polA:=gcd(polA,newpolA);
  fi;
elif polB<>0 then
  newpolA:=resultant(polB,listAB[1],B);
  polA:=gcd(polA,newpolA);
  if nopsAB>=3 then
      newpolA:=resultant(listAB[2],listAB[3],B);
      polA:=gcd(polA,newpolA);
  fi;
fi;
newpolA:='newpolA';

# We have constructed a polynomial polA<>0.
# we consider it factors and do calculations with them.

factorspolA:=factors(polA); 
numbfactpolA:=nops(factorspolA[2]);
lprint(`    A first polynomial for `, var1,` is:`,factor(polA));
# lprint();
if degree(polA,A)=0 and polA<>0 then
  lprint(`  --There are NO solutions for `, var1,` and `, var2);
  # lprint();
  RETURN(listsolutions);
else 
  lprint(`    Its number of (non-constant) factors is:`,numbfactpolA);
  lprint(`    For each factor we determine an equation in `,B,` using gcd's and resultants.`);
  # lprint();
fi;

# Make a polynomial for B per factor.

for k from 1 to numbfactpolA do 
  factpolA:=factorspolA[2][k][1];
  factpolA:=integercoeffpol(factpolA);
  alpha:=RootOf(factpolA,A);
  listalphaB:=[];
  for i from 1 to nopsAB do
    substeqn:=evala(subs(A=alpha,listAB[i])); 
    listalphaB:=[op(listalphaB),substeqn];
  od;
  substeqn:='substeqn'; 

  eqnBalpha:=0; 
  for i from 1 to nopsAB do
    eqnBalpha:=evala(Gcd(eqnBalpha,listalphaB[i]));
  od;
  eqnBalpha:=evala(Gcd(eqnBalpha,polB)); 
  if type(alpha,rational)=false then
    # Hopefully the following factorization indeed happens over Q(alpha).
    eqnBalpha:=factor(eqnBalpha,alpha);  
  fi;
  eqnBalpha:=(subs(alpha=A,eqnBalpha));  
  
  if degree(eqnBalpha,B)=0 and eqnBalpha<>0 then
    lprint(`  --There are NO solutions for the polynomial factor nr.`, k);
    # lprint();
  elif eqnBalpha=0 then
    listsolutions:=[op(listsolutions),[factpolA,0,0]];
    # lprint(`    --The unknown`, var2,` is arbitrary for factor nr.`, k);
    lprint(`  --Factor nr `,k,` leads to the solution.`);
    print(listsolutions[nops(listsolutions)]);
    # lprint();
  else

# We have now a polynomial for B over Q(alpha) of degree >=1 in B.

    factorspolAB:=factors(eqnBalpha); 
    numbfactpolAB:=nops(factorspolAB[2]);
 
    for m from 1 to numbfactpolAB do 
      factpolAB:=factorspolAB[2][m][1];
      factpolAB:=integercoeffpol(factpolAB);    
      eqnBrational:=resultant(factpolA,factpolAB,A);
      # eqnBrational:=resultant(factpolA,listAB[1],A); 
      eqnBrational:=gcd(polB,eqnBrational); 
      i:=1;
      while eqnBrational=0 and nopsAB>1 do
        eqnBrational:=resultant(factpolA,listAB[i],A);
        i:=i+1;
      od;
      i:='i';
      eqnBrational:=convertpolintsquarefree(factors(eqnBrational));
      factorspolB:=factors(eqnBrational); 
      for s from 1 to nops(factorspolB[2]) do
        factpolB:=factorspolB[2][s][1];
        factpolB:=integercoeffpol(factpolB); 
        listsolutions:=[op(listsolutions),[factpolA,factpolB,subs(A=var1,factpolAB)]];
        lprint(`  --Factor nr `,k,` leads to the solution.`);
        print(listsolutions[nops(listsolutions)]); 
        # lprint();   
      od;
    od;


  fi;
od;
k:='k';

RETURN(listsolutions);

end:




##########################
##########################
# Procedures for monodromy group G422.
##########################
##########################



with(linalg):
lameG422:=proc(n)  #  Determines Rn(X) for n given.

local d,epsilon,M,k,i,Rn;

if type(n-1/2,nonnegint)=false then
  ERROR(`The variable n must be a in 1/2+Z and >0.`);
fi;

d:=n+1/2;   
epsilon:=-n/2;

M:=matrix(d,d);

for k from 1 to d do  
  for i from 1 to (k-3) do
    M[k,i]:=0; 
  od;
  for i from k+2 to d do
    M[k,i]:=0;
  od;
  if type(k-2,posint)=true then
    M[k,k-2]:=g3*(k-2+epsilon)*(k-3+epsilon);
  fi;
  if type(k-1,posint)=true then
    M[k,k-1]:=g2*(k-3/2+epsilon)*(k-2+epsilon);
  fi;
  M[k,k]:=X;
  if type(d-k,posint)=true then
    M[k,k+1]:=-4*k*(k-1/2+2*epsilon);
  fi;
od;
# print(evalm(M));

Rn:=simplify(det(M));
Rn:=sort(Rn,X);

print(`The list [n,Rn] is:`);

RETURN([n,Rn]);
end:




##########################
##########################
# Procedures for monodromy group DN.
##########################
##########################




dihedralinput:=proc(n::nonnegint,N::nonnegint,B::complex) # checks and fixes 
                                                          # input.
  # Output: n,N,B

if N<3 then
  ERROR(`The second variable N must be a positive integer >= 3.`);
fi;

RETURN(n,N,B);

end:



##########################



lamePn:=proc(n::nonnegint)    
                       # Determines the polynomial Pn(z) of degree n, that is 
                       # under the monodromy action of DN.

local orders1,k,s1,u,seriess1,squareseriess1,s1t2,invpol:

# We have y1(t)=:=t^(-n/2)*s1 .
# The power series s1 needs to be determines upto degree n in t.

orders1:=n;  

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders1 do
  u(k):=simplify(B*u(k-1)+g2*(k-2-n/2)*(k-3/2-n/2)*u(k-2)+\
           g3*(k-3-n/2)*(k-2-n/2)*u(k-3));
  u(k):=simplify(u(k)/(4*k*(k-n-1/2)) );
od;
s1:=sum('u(k)*t^(k)','k'=0..orders1); 
u:='u';
#lprint(`----The expansion of s1 belonging to the exponent -n/2 in t around t=0`);
#lprint(`    has been determined upto degree`, orders1);

seriess1:=series(s1,t,n+1);
squareseriess1:=series(seriess1^2,t,n+1);
s1t2:=convert(squareseriess1,polynom);

invpol:=t^(-n)*s1t2;
invpol:=collect(simplify(invpol),t);
invpol:=collect(subs(t=1/z,invpol),z);
invpol:=sort(invpol,z);
#lprint(`----The polynomial P_n in z=1/t is:`);

RETURN(invpol);

end:



##########################



seriess12andgamma:=proc(n::nonnegint,N::nonnegint,B,g2,g3)  
                              # Determines the local series around infinity
                              # in the local parameter t correctly up to degree 
                              # 4 in t, and gamma^2.
                              # Output: s1(t), s2(t), gamma^2.

local orders1,orders2,k,s1t,u,seriess1, \
      squareseriess1,s1t2,Pn,squareofgamma,s2t:

orders1:=max(ceil(n*N/2)+4,2*n+1);   # 2n+1 comes from determination of gamma.
orders2:=ceil(n*N/2-n+7/2);          # This is ceil(nN/2+4 - (n+1/2)).

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders1 do
  u(k):=simplify(B*u(k-1)+g2*(k-2-n/2)*(k-3/2-n/2)*u(k-2)+\
           g3*(k-3-n/2)*(k-2-n/2)*u(k-3));
  u(k):=simplify(u(k)/(4*k*(k-n-1/2)) );
od;
s1t:=sum('u(k)*t^(k)','k'=0..orders1); 
u:='u';

seriess1:=series(s1t,t,2*n+2); 
squareseriess1:=series(seriess1^2,t,2*n+2);
s1t2:=convert(squareseriess1,polynom);
s1t2:=collect(simplify(s1t2),t);

squareofgamma:=coeff(s1t2,t,2*n+1);
squareofgamma:=factor(squareofgamma);

# Calculate s2 by using the resursive relation.

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders2 do
  u(k):=simplify(B*u(k-1)+g2*(k-3/2+n/2)*(k-1+n/2)*u(k-2)+\
           g3*(k-3/2+n/2)*(k-5/2+n/2)*u(k-3));
  u(k):=simplify(u(k)/(4*k*(k+n+1/2)) );
od;
s2t:=sum('u(k)*t^(k)','k'=0..orders2); 
u:='u';

#lprint(` s1,s2,gamma^2 =`);

RETURN(s1t,s2t,squareofgamma);

end:



##########################



seriess12DN:=proc(n::nonnegint,N::nonnegint,B,g2,g3,orderseriess1,orderseriess2)  
                              # Determines the local series around infinity
                              # in the local parameter t correctly up to the given 
                              # degrees in t, if given. Otherwsie it uses the 
                              # default settings.
                              # Output: s1(t), s2(t).

local orders1,orders2,optionalinput,definedorders1,definedorders2,k,s1t,u,seriess1, \
      squareseriess1,s1t2,Pn,squareofgamma,s2t:

orders1:=max(ceil(n*N/2)+4,2*n+1);   # 2n+1 comes from determination of gamma.
orders2:=ceil(n*N/2-n+7/2);          # This is ceil(nN/2+4 - (n+1/2)).

if nargs>5 then
  optionalinput := select(type,{args},`=`); 
  definedorders1:=subs(optionalinput,'orderofs1'); 
  definedorders2:=subs(optionalinput,'orderofs2'); 
  if type(definedorders1,'integer') then
    orders1:=definedorders1;
  fi;
  if type(definedorders2,'integer') then  
    orders2:=definedorders2;
  fi;
fi;

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders1 do
  u(k):=simplify(B*u(k-1)+g2*(k-2-n/2)*(k-3/2-n/2)*u(k-2)+\
           g3*(k-3-n/2)*(k-2-n/2)*u(k-3));
  u(k):=simplify(u(k)/(4*k*(k-n-1/2)) );
od;
s1t:=sum('u(k)*t^(k)','k'=0..orders1); 
u:='u';

# Calculate s2 by using the resursive relation.

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders2 do
  u(k):=simplify(B*u(k-1)+g2*(k-3/2+n/2)*(k-1+n/2)*u(k-2)+\
           g3*(k-3/2+n/2)*(k-5/2+n/2)*u(k-3));
  u(k):=simplify(u(k)/(4*k*(k+n+1/2)) );
od;
s2t:=sum('u(k)*t^(k)','k'=0..orders2); 
u:='u';

RETURN(s1t,s2t);

end:



##########################


oldsquareofgamma:=proc(n::nonnegint,N::nonnegint,B,g2,g3,s1)

local seriess1, squareseriess1,s1t2,thesquareofgamma:

if degree(s1,t)<2*n+1 then
  lprint(`----The degree of s1 in t must be at least 2n+1.`);
  RETURN();
fi;

seriess1:=series(s1,t,2*n+2); 
squareseriess1:=series(seriess1^2,t,2*n+2);
s1t2:=convert(squareseriess1,polynom);
s1t2:=collect(simplify(s1t2),t);

thesquareofgamma:=coeff(s1t2,t,2*n+1);
thesquareofgamma:=factor(thesquareofgamma);

RETURN(thesquareofgamma);

end:



##########################



squareofgamma:=proc(n::nonnegint,B,g2,g3)

local orders1, k, u, s1t, seriess1, squareseriess1,s1t2,thesquareofgamma:

orders1:=2*n+1;

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders1 do
  u(k):=simplify(B*u(k-1)+g2*(k-2-n/2)*(k-3/2-n/2)*u(k-2)+\
           g3*(k-3-n/2)*(k-2-n/2)*u(k-3));
  u(k):=simplify(u(k)/(4*k*(k-n-1/2)) );
od;
s1t:=sum('u(k)*t^(k)','k'=0..orders1); 
u:='u';

seriess1:=series(s1t,t,2*n+2); 
squareseriess1:=series(seriess1^2,t,2*n+2);
s1t2:=convert(squareseriess1,polynom);
s1t2:=collect(simplify(s1t2),t);

thesquareofgamma:=coeff(s1t2,t,2*n+1);
thesquareofgamma:=factor(thesquareofgamma);

RETURN(thesquareofgamma);

end:



##########################



invariantdegN:=proc(n,N,B,s1t,s2t,thegamma) # Calculates (f1^N-gamma^2f2^N)(s)
                                            # with s^2=t.
  # Output: (f1^N-gamma^2f2^N)(s) as a Laurent series in s of degree about 8.

local s1s,s2s,\
      orderseriess,g1s,seriesg1s,seriesg1sN,g1sN,g2s,seriesg2s,\
      seriesg2sN,g2sN,almostinv,invariantseries,degreeinvs:

s1s:=subs(t=s^2,s1t); 
s2s:=subs(t=s^2,s2t); 
orderseriess:=n*N+8;

# We write g1s for s1s+gamma*s^(2n+1)s2s. So we have f1=s^(-n)*g1s.
# Analogously we write f2=s^(-n)*g2s.
# This gives f1^N=s^(-nN)*g1s^N and f2^N=s^(-nN)*g2s^N.
# The N-th powers of g1s and g2s upto degree degsumN+8 in s will
# be denoted by g1sN and g2sN, respectively.
# Notice that we then have f1^N+f2^N=s^(-nN)*(g1sN+g2sN).

# Determine the N-th powers of g1 and g2 and g1^N+g2^N in s:

g1s:=collect(s1s+thegamma*s^(2*n+1)*s2s,s); 
seriesg1s:=series(g1s,s,orderseriess+1);
seriesg1sN:=series(seriesg1s^N,s,orderseriess+1);
g1sN:=convert(seriesg1sN,polynom);
lprint(`The first N-th power has been determined.`);

g2s:=collect( (-1)^n*(s1s-thegamma*s^(2*n+1)*s2s),s);
seriesg2s:=series(g2s,s,orderseriess+1);
seriesg2sN:=series(seriesg2s^N,s,orderseriess+1);
g2sN:=convert(seriesg2sN,polynom);
lprint(`The second N-th power has been determined`);

almostinv:=simplify(collect(g1sN+g2sN,s));
almostinv:=factorcoeffspol(almostinv,s,degree(almostinv,s));
g1s:='g1s';
g2s:='g2s';

# Determine invariant:=f1^N+f2^N in s:

invariantseries:=sort(collect(s^(-n*N)*almostinv,s),s);
degreeinvs:=degree(invariantseries,s);

#if n*N<+12 then 
#  lprint(`The invariant f1^N+f2^N up to degree`, degreeinvs, `in s is:`);
#  print(invariantseries );
#else
#  lprint(`The series of f1^N+f2^N up to degree`, degreeinvs, `in s is determined.`);
#fi;

lprint(`The series of f1^N+f2^N up to degree`, degreeinvs, `in t = s^2 is determined.`);

RETURN(invariantseries);

end:



##########################



firsteqnsDN:=proc(n,N,B,squareofgamma,invariantN) # puts the coefficients of the
                                                # the positive powers of s in
                                                # in the series invariantN in
                                                # a list after factoring out
                                                # all appearing factors of
                                                # gamma.
                                                # Output: This list.

local firstlistgamma,numbfactsgamma,listfactsgamma,i,newinvariant,eqn,\
      listeqns,j,setofsolarrays,\
      localsetsols,setgamma,setnewgamma,\
      nopslocal,alpha,setABs:

# Make a list of the factors of gamma^2 with integer coefficients:

firstlistgamma:=factors(squareofgamma)[2];
numbfactsgamma:=nops(firstlistgamma) ; 
listfactsgamma:=[];
for i from 1 to numbfactsgamma do
  listfactsgamma:=[op(listfactsgamma),integercoeffpol(firstlistgamma[i][1])]; 
od;
firstlistgamma:='firstlistgamma'; 

# Determine the four polynomial coefficients and make them squarefree
# with integral coefficients and having no factors of gamma: 

newinvariant:=invariantN; 

listeqns:=[];   
for i from 1 to 4 do
  if type(n*N,even)=true then
    eqn:=factors(evala(coeff(newinvariant,s^(2*i)))); 
  else  
    eqn:=factors(evala(coeff(newinvariant,s^(2*i))^2));  
    # to get rid of the possible root.
  fi;
  eqn:=convertpolintsquarefree(eqn); 
  if eqn<>0 then 
    listeqns:=[op(listeqns),eqn];    
  fi;
od;
eqn:='eqn';   

for i from 1 to nops(listeqns) do
  for j from 1 to numbfactsgamma do
    while type(normal(listeqns[i]/listfactsgamma[j]),polynom)=true do 
      listeqns:=subsop(i=normal(listeqns[i]/listfactsgamma[j]),listeqns); 
    od;
  od;
od;   

RETURN(listeqns);

end:



##########################  



checkdiscriminant:=proc(listofpols,X,Y) #  is the same as the one given later

local localsetsols,nopslocal,polpXY,i,alpha,beta,setsubs1,setsubs2,\
      subsp,discrimsubsp,newsetpols:

localsetsols:=listofpols;
nopslocal:=nops(localsetsols);

polpXY:=4*z^3-X*z-Y;
newsetpols:=[];

for i from 1 to nopslocal do
  if nops(localsetsols[i])>0 then 
    if localsetsols[i][1]<>X or localsetsols[i][2]<>Y then
      if localsetsols[i][1]<>0 and localsetsols[i][3]=0 then 
        alpha:=RootOf(localsetsols[i][1],X);  
        setsubs1:={X=alpha};
        setsubs2:={};
      elif localsetsols[i][1]=0 and localsetsols[i][3]<>0 then
        beta:=RootOf(localsetsols[i][3],Y);
        setsubs1:={Y=beta};
        setsubs2:={};
      elif localsetsols[i][1]<>0 and localsetsols[i][3]<>0 then
        alpha:=RootOf(localsetsols[i][1],X);
        beta:=RootOf(localsetsols[i][3],Y);
        setsubs1:={Y=beta}; 
        setsubs2:={X=alpha,g2=alpha};
      fi;

      subsp:=subs(setsubs1,polpXY);
      subsp:=subs(setsubs2,subsp);   
      subsp:=evala(Factor(subsp));  
      discrimsubsp:=evala(discrim(subsp,z));  

      if discrimsubsp<>0 then
        newsetpols :=[op(newsetpols),localsetsols[i]];
      fi;
    fi;
  fi;
od;

RETURN(newsetpols);
   
end;



##########################



polynomialcheck:=proc(n,N,B,g2,g3,s1,invariant,listofpols,X,Y) 

local localsetsols,nopslocal,s1XY,localinvariant,polpXY,testsolutionset,\
      solutionset,i,alpha,beta,setsubs1,setsubs2,revset,revsetg2g3,\
      subsp,discrimsubsp,localinv,As,almostAs,Az,locals1,squarelocals1,\
      almostPn,Pn,subsPn,\
      almostBz,Bz2,factorsBz,Bz,k,testsolution,solution:

localsetsols:=listofpols; print(%);
nopslocal:=nops(localsetsols);
s1XY:=subs({g2=X,g3=Y},s1);
localinvariant:=subs({g2=X,g3=Y},invariant); 
polpXY:=4*z^3-X*z-Y;
testsolutionset:=[];
solutionset:=[];

for i from 1 to nopslocal do
  if nops(localsetsols[i])>0 then
    if localsetsols[i][1]<>X or localsetsols[i][2]<>Y then
      if localsetsols[i][1]<>0 and localsetsols[i][3]=0 then 
        alpha:=RootOf(localsetsols[i][1],X);  
        setsubs1:={X=alpha};
        setsubs2:={};
        revset:={alpha=X};
        revsetg2g3:={alpha=g2,Y=g3};
      elif localsetsols[i][1]=0 and localsetsols[i][3]<>0 then
        beta:=RootOf(localsetsols[i][3],Y);
        setsubs1:={Y=beta};
        setsubs2:={};
        revset:={beta=Y};
        revsetg2g3:={beta=g3,X=g2};
      elif localsetsols[i][1]<>0 and localsetsols[i][3]<>0 then
        alpha:=RootOf(localsetsols[i][1],X);
        beta:=RootOf(localsetsols[i][3],Y);
        setsubs1:={Y=beta}; 
        setsubs2:={X=alpha,g2=alpha};
        revset:={beta=Y,alpha=X};
        revsetg2g3:={alpha=g2,beta=g3};
      fi;
      

      subsp:=subs(setsubs1,polpXY);
      subsp:=subs(setsubs2,subsp);   
      subsp:=evala(Factor(subsp));  
      discrimsubsp:=evala(discrim(subsp,z));  

      if discrimsubsp<>0 then

        # construct the polynomial A(z).

        localinv:=evala(Expand(subs(setsubs1,localinvariant)));
        localinv:=evala(Expand(subs(setsubs2,localinv)));  
                                               # This is an inportant move:
                                               # Do sustitution in 2 parts.
        As:=(1/2)*localinvariant;
        almostAs:=collect(As,s);
        almostAs:=series(almostAs,s,1);
        almostAs:=convert(almostAs,polynom);
        almostAs:=subs(s=1/squarez,almostAs); 
        almostAs:=collect(almostAs,squarez); 
        Az:=collect(subs(squarez=sqrt(z),almostAs),z); 
        Az:=evala(Expand(subs(setsubs1,Az)));  
        Az:=evala(Expand(subs(setsubs2,Az)));
        Az:=sort(Az,z); 
        # print(`Az is`,%);

        # construct the polynomial Pn for our particular case.
      
        locals1:=series(s1XY,t,2*n+2); 
        locals1:=evala(Expand(subs(setsubs1,locals1))); 
        locals1:=evala(Expand(subs(setsubs2,locals1))); 

        squarelocals1:=evala(series(locals1^2,t,n+1)); 
        almostPn:=convert(squarelocals1,polynom);
        almostPn:=collect(simplify(almostPn),t); 
        Pn:=t^(-n)*almostPn;
        Pn:=collect(simplify(Pn),t);
        Pn:=series(Pn,t,1); 
        Pn:=convert(Pn,polynom);
        Pn:=collect(subs(t=1/z,Pn),z); 
        # print(`Pn is`, %);

        subsPn:=evala(Expand(subs(setsubs1,Pn))); 
        subsPn:=evala(Expand(subs(setsubs2,subsPn)));

        # The function 4*Az^2-(-1)^(n*N)*4*subsPn^N should be p(z)*4*B^2(z)
        # We determine B(z) (up to the sign +-).

        almostBz:=4*Az^2-(-1)^(n*N)*4*subsPn^N;   
        almostBz:=evala(Expand(almostBz)); 
        almostBz:=evala(Factor(almostBz));  
        almostBz:=sort(almostBz,z); 
        if evala(Divide(almostBz,subsp))=true then  
          almostBz:=evala(Quo(almostBz,subsp,z));
          almostBz:=evala(Factor(almostBz));
        else
          almostBz:=0;
        fi;
        Bz2:=almostBz/4;
        # Bz:=sqrt(almostBz^2); 
        Az:=subs(revsetg2g3,Az); 
        Az:=simplify(Az);
        # Az:=factor(Az);
        Bz2:=subs(revsetg2g3,Bz2); 
        Bz2:=simplify(Bz2);
        factorsBz:=factors(Bz2); 
        Bz:=sqrt(factorsBz[1]); 
        for k from 1 to nops(factorsBz[2]) do       
          Bz:=Bz*(factorsBz[2][k][1])^(factorsBz[2][k][2]/2);
        od; 
        if Bz<>0 then
            testsolution:=[op(localsetsols[i]),Az,Bz]; 
            testsolutionset:=[op(testsolutionset),testsolution];
        fi;
      fi;
    fi;
  fi;
od;

RETURN(testsolutionset);

end;



##########################



valuelameoperatorDN:=proc(n,N,B,g2,g3,X,Y,fX,fY,fYg2,Az,Bz)
             # output: Ln(y), with y=(A(z)+sqrt(p)B(z))^(1/N). 

local rootfX,rootfYX,localp,localrootp,dlocalp,localAz,localBz,\
      phi,dphi,ddphi,Ln1,Ln2,Ln;

Ln:=1;

if fX<>0 then
  rootfX:=RootOf(fX,X); 
else
  rootfX:=X;
fi;
 
if fYg2<>0 then
  rootfYX:=RootOf(subs(g2=rootfX,fYg2),Y);
else
  rootfYX:=Y; 
fi;
# print(fYg2,rootfX,rootfYX);

localp:=4*z^3-rootfX*z-rootfYX;  
localrootp:=RootOf(T^2-localp,T); 
dlocalp:=diff(localp,z);  

localAz:=subs(g3=rootfYX,Az); 
localAz:=evala(Expand(localAz)); 
localAz:=subs(g2=rootfX,localAz);
localAz:=evala(Expand(localAz));

localBz:=subs(g3=rootfYX,Bz);
localBz:=evala(Expand(localBz));
localBz:=subs(g2=rootfX,localBz);
localBz:=evala(Expand(localBz)); 
#print(localAz,localBz);

phi:=localAz+localrootp*localBz;
dphi:=diff(phi,z);
dphi:=evala(Normal(dphi));
dphi:=evala(Expand(numer(dphi)))/denom(dphi); 

ddphi:=diff(dphi,z);
ddphi:=evala(Normal(ddphi)); 
ddphi:=evala(Expand(numer(ddphi)))/denom(ddphi);

Ln1:=localp*(dphi^2*(1/N^2-1/N)+ddphi*phi/N); 
Ln1:=evala(Normal(Ln1));
Ln1:=evala(Expand(numer(Ln1)))/denom(Ln1);

Ln2:=1/2*dlocalp*dphi*phi/N-(n*(n+1)*z+B)*phi^2;
Ln2:=evala(Normal(Ln2)); 
Ln2:=evala(Expand(numer(Ln2)))/denom(Ln2); 

Ln:=evala(Normal(Ln1+Ln2)); 
Ln:=evala(Expand(numer(Ln)))/denom(Ln); 

Ln:=evala(Expand(denom(Ln)*numer(Ln)));
Ln:=subs({g3=rootfYX,g2=rootfX},Ln);
Ln:=simplify(Ln);

RETURN(Ln);

end:




##########################################################################




lameDN:=proc(n,N,B)

local g2,g3,s1,s2,squaregamma,gamma,lameDNinput,invariant,
      startinglistg2g3,listeqns,X,Y,firstlistpols,secondlistpols,
      validlistpols,listallpols,nopsvalid,i,L,resultlameeqn:
                                             
#check the input.

lameDNinput:=dihedralinput(n,N,B);

# define series around infinity.

s1,s2,squaregamma:=seriess12andgamma(lameDNinput,g2,g3);

# display the term gamma^2.
# if it is 0 we are done.

lprint(`The square of gamma is:`);
print(squaregamma);
if squaregamma=0 then
  print();
  print(` ******************************************************************`);
  print(`The list of all algebraic Lame' equations with n, N, B =`, lameDNinput);
  print(`is given by the following list with entries`); 
  print(`[n, N, B, pol(g2)/Q, pol(g3)/Q, pol(g3)/Q(g2), A(z), B(z)].`);

  RETURN([]);
fi;

# define gamma as a square root and calculate the invariant of
# degree N under DN in g2 and g3.
# make a new invariant in which we substitute g2=X and g3=Y.

gamma:=RootOf(T^2-squaregamma,T);
invariant:=invariantdegN(lameDNinput,s1,s2,gamma);

# get the first polynomial coefficients.
 
startinglistg2g3:=firsteqnsDN(lameDNinput,squaregamma,invariant); 
listeqns:=subs({g2=X,g3=Y},startinglistg2g3);

# solve the system of polynomials.

lprint(`We determine the list of polynomial equations g2 and g3 must satisfy`);
firstlistpols:=reducebivarpols(listeqns,g2,g3,X,Y); 

# test if the polynomials give an algebraic Lame equation.

validlistpols:=polynomialcheck(lameDNinput,g2,g3,s1,invariant,firstlistpols,X,Y);

listallpols:=[];
nopsvalid:=nops(validlistpols);
for i from 1 to nopsvalid do
  listallpols:=[ op(listallpols), [validlistpols[i][1], validlistpols[i][2],
                validlistpols[i][3], subs({X=g2,Y=g3},validlistpols[i][4]),
                subs({X=g2,Y=g3},validlistpols[i][5])] ];
od; 
lprint(`The following list of candidate solutions\
 [pol(g2)/Q, pol(g3)/Q, pol(g3)/Q(g2),A(z),B(z)] as entries`);
lprint(`need to be checked as being solutions of the Lame' equation`);
print(listallpols);

# Construct the list of solutions L after verifying that they satify the 
# Lame equation.

lprint(`Checking the candidate solutions....`);
L:=[];
for i from 1 to nopsvalid do
  resultlameeqn:=valuelameoperatorDN(lameDNinput,g2,g3,X,Y,op(listallpols[i]));
  if resultlameeqn=0 then
    L:=[ op(L) , [lameDNinput,op(listallpols[i])] ];
  fi;
od;

print();
print(` ******************************************************************`);
print(`The list of all algebraic Lame' equations with n, N, B =`, lameDNinput);
print(`is given by the following list with entries`); 
print(`[n, N, B, pol(g2)/Q, pol(g3)/Q, pol(g3)/Q(g2), A(z), B(z)].`);

RETURN(L);

end;




##########################
##########################
# Procedures for the monodromy group G12, G13 and G22.
##########################
##########################




inputG:=proc(M,n::rational,B::complex) 
  # checks and fixes input.
  # Output: M,n,B

if M<>G12 and M<>G13 and M<>G22 then
  ERROR(`The first variable (the group) must be G12, G13 or G22.`);
fi;

if n<-1/2 then
  ERROR(`The second variable n must at least be -1/2 .`);
fi;

if M=G12 then
  if type(2*n,integer)=true or type(4*n,integer)=false then
    ERROR(`The second variable n must be 1/4 or -1/4 modulo Z.`);
  fi;
fi;

if M=G13 then
  if type(2*n,integer)=true or type(3*n,integer)=true or 
    type(6*n,integer)=false then
    ERROR(`The second variable n must be 1/6 or -1/6 modulo Z.`);
  fi;
fi;

if M=G22 then
  if type(2*n,integer)=true or type(5*n,integer)=true or 
    type(10*n,integer)=false then
    ERROR(`The second variable n must be 1/10, 3/10, -1/10 or -3/10 modulo Z.`);
  fi;
fi;

RETURN(M,n,B);

end:



##########################


inputG22:=proc(n::rational,B::complex) 
  # checks and fixes input.
  # Output: n,N,B

if n<-1/2 then
  ERROR(`The first variable n must at least be -1/2 .`);
fi;

if type(2*n,integer)=true or type(3*n,integer)=true or 
   type(6*n,integer)=false then
     ERROR(`The second variable n must be 1/6 or 5/6 modulo Z.`);
fi;

RETURN(n,B);

end:



##########################



calculatem:=proc(M,n::rational) # calculates the number m for the
      # given group m, and its degrees dgamma and dm.
      # output: m,dgamma,dm

local m,dgamma,dm:

if M=G12 then
  m:=6;
elif M=G13 then
  m:=8;
else
  m:=12;
fi;

dgamma:=(m-2)*n/2+(m-1)/2; 
dm:=n*m/2-n-1/2;

RETURN(m,dgamma,dm);

end:



##########################



seriess12G:=proc(M,n,B,g2,g3,orderseriess1,orderseriess2)  
                              # Determines the local series around infinity
                              # in the local parameter t correctly up to the given
                              # degrees in t, if given. Otherwise it uses the 
                              # default settings.
                              # Output: s1(t), s2(t).

local m, dm, orders1,orders2,optionalinput,definedorders1,definedorders2,\
      k,s1,u,s2:

if M=G22 and type(6*n,integer)=true then
  orders1:=6*n+8;   
  orders2:=3*n-3/2+8; 
else
  if M=G12 then
    m:=6;
  elif M=G13 then
    m:=8;
  else
    m:=12;
  fi;
  dm:=n*m/2-n-1/2; 

  orders1:=ceil(dm+5);   
  orders2:=ceil(dm+5);
fi; 

if nargs>5 then 
  optionalinput := select(type,{args},`=`); 
  definedorders1:=subs(optionalinput,'orderofs1'); 
  definedorders2:=subs(optionalinput,'orderofs2'); 
  if type(definedorders1,'integer') then
    orders1:=definedorders1;
  fi;
  if type(definedorders2,'integer') then  
    orders2:=definedorders2;
  fi;
fi;

# Calculate s1 by using the resursive relation.

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders1 do
  u(k):=simplify(B*u(k-1)+g2*(k-2-n/2)*(k-3/2-n/2)*u(k-2)+\
           g3*(k-3-n/2)*(k-2-n/2)*u(k-3));
  u(k):=simplify(u(k)/(4*k*(k-n-1/2)) );
od;
s1:=sum('u(k)*t^(k)','k'=0..orders1); 
u:='u';

# Calculate s2 by using the resursive relation.

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders2 do
  u(k):=simplify(B*u(k-1)+g2*(k-3/2+n/2)*(k-1+n/2)*u(k-2)+\
           g3*(k-3/2+n/2)*(k-5/2+n/2)*u(k-3));
  u(k):=simplify(u(k)/(4*k*(k+n+1/2)) );
od;
s2:=sum('u(k)*t^(k)','k'=0..orders2); 
u:='u';

RETURN(s1,s2);

end:



##########################



listcoefsG:=proc(M,n,B,g2,g3,s1,s2) # produces a list of the coefficients of
    # Jm(y1,y2). The number we taken is <=4 for M=G12 and <=5 for M=G13.
    # In the case of M=G22 we take 5 equations.
    # Output: L,beta
    # where L is the list of coefficients and beta the constant in Q[g2,g3]
    # appearing in Jm(X,Y). 

local m,dgamma,dm,lowestseriess1,lowestseriess1s2,su,\
      dsv,sndseriess1,sndseriess2,sndseriess1s2,sv,\
      L,beta,gamma,j,listE,i:

if M=G12 then
  m:=6;
elif M=G13 then
  m:=8;
else
  m:=12;
fi;

dgamma:=(m-2)*n/2+(m-1)/2;
dm:=n*m/2-n-1/2;

# calculate su:= y1^(m-1)*y2(t) up to degree <=5.

lowestseriess1:=series(s1,t,dm+5+1);
lowestseriess1:=series(lowestseriess1^(m-1),t,dm+5+1);
lowestseriess1s2:=series(lowestseriess1*s2,t,dm+5+1);
lowestseriess1s2:=convert(lowestseriess1s2,polynom);
lowestseriess1s2:=collect(simplify(lowestseriess1s2),t);
su:=collect(t^(-dm)*lowestseriess1s2,t); 
# print(%);

# calculate sv:= y1^m/2*y2^m/2(t) up to degree 5

if M<>G12 then   
  dsv:=m/4;
  sndseriess1:=series(s1,t,5-dsv+1);
  sndseriess1:=series(sndseriess1^(m/2),t,5-dsv+1);
  sndseriess2:=series(s2,t,5-dsv+1);
  sndseriess2:=series(sndseriess2^(m/2),t,5-dsv+1);
  sndseriess1s2:=series(sndseriess1*sndseriess2,t,5-dsv+1);
  sndseriess1s2:=convert(sndseriess1s2,polynom);
  sndseriess1s2:=collect(simplify(sndseriess1s2),t);
  sv:=collect(t^dsv*sndseriess1s2,t); 
  # print(%);
fi;

# Make the list of equations and calculate beta.

L:=[];


if M=G12 then
  beta:=0;  
   # gamma:=-coeff(su,t,dgamma);
  j:=min(dgamma-1,4); 
  for i from 1 to j do
    listE[i]:=coeff(su,t,i); 
    listE[i]:=integercoeffpol(listE[i]);
    # listE[i]:=simplifypoleqn(listE[i]);
  od;
  listE:=[seq(listE[i],i=1..j)]; 
  j:='j';

elif M=G13 then
  beta:=-coeff(su,t,2);  
   # gamma:=-coeff(su,t,dgamma)-beta*coeff(sv,t,dgamma);
  listE[1]:=coeff(su,t,1); 
  listE[1]:=integercoeffpol(listE[1]);
  j:=min(dgamma-1,5); 
  for i from 2 to j-1 do
    listE[i]:=coeff(su,t,i+1)+beta*coeff(sv,t,i+1); 
    listE[i]:=expand(listE[i]); 
    listE[i]:=integercoeffpol(listE[i]);
  od;
  listE:=[seq(listE[i],i=1..j-1)]; 
  j:='j';

else
  beta:=-coeff(su,t,3);  
   # gamma:=-coeff(su,t,dgamma)-beta*coeff(sv,t,dgamma); 
  for i from 1 to 2 do
    listE[i]:=coeff(su,t,i); 
    listE[i]:=integercoeffpol(listE[i]);
  od;
  for i from 3 to 4 do
    listE[i]:=coeff(su,t,i+1)+beta*coeff(sv,t,i+1); 
    listE[i]:=expand(listE[i]); 
    listE[i]:=integercoeffpol(listE[i]);
  od;
  listE:=[seq(listE[i],i=1..4)]; 
fi;

RETURN(listE,beta);

end:



##########################



listcoefsG22:=proc(n,B,g2,g3,s1,s2,inputa9) 
    # produces a list of the coefficients of
    # Jm(y1,y2). The number we taken is <=4 for n=-1/2 and 6 otherwise.
    # Output: L, [a9,a6,a3]
    # where L is the list of coefficients appearing in J12(X,Y). 

local orders1,orders2,seriessu12,su12,seriessu09,su09,seriessu06,su06,\
      su03,seriessu03,su00,seriessu00,a9,a6,a3,a0,c2,c4,c6,c8,C,c1,c5,c7,\
      listofa:

orders1:=degree(s1,t);
orders2:=degree(s2,t);

# calculate su12:= y1^12 up to degree 8.

seriessu12:=series(s1,t,orders1+1);
seriessu12:=series(seriessu12^(12),t,orders1+1);
seriessu12:=convert(seriessu12,polynom);
seriessu12:=collect(simplify(seriessu12),t);
su12:=collect(t^(-6*n)*seriessu12,t); 

# calculate su09:= y1^9y2^3 up to degree 8.

seriessu09:=series(s1,t,3*n-3/2+8+1);
seriessu09:=series(seriessu09^(9),t,3*n-3/2+8+1);
seriessu09:=series(seriessu09*s2^(3),t,3*n-3/2+8+1);
seriessu09:=convert(seriessu09,polynom);
seriessu09:=collect(simplify(seriessu09),t);
su09:=collect(t^(-3*n+3/2)*seriessu09,t);
 
# calculate su06:= y1^6y2^6 up to degree 8.

seriessu06:=series(s1,t,-3+8+1);
seriessu06:=series(seriessu06^(6),t,-3+8+1);
seriessu06:=series(seriessu06*s2^(6),t,-3+8+1);
seriessu06:=convert(seriessu06,polynom);
seriessu06:=collect(simplify(seriessu06),t);
su06:=collect(t^(3)*seriessu06,t); 

# calculate su03:= y1^3y2^9 up to degree 8. 
# If -3*n-9/2+8+1>=0 the series is of importance for the coefficients.

if -3*n-9/2+8+1<0 then
  su03:=0;
else
  seriessu03:=series(s1,t,-3*n-9/2+8+1); 
  seriessu03:=series(seriessu03^(3),t,-3*n-9/2+8+1);
  seriessu03:=series(seriessu03*s2^(9),t,-3*n-9/2+8+1);
  seriessu03:=convert(seriessu03,polynom);
  seriessu03:=collect(simplify(seriessu03),t);
  su03:=collect(t^(3*n+9/2)*seriessu03,t); 
fi;

# calculate su00:= y2^12 up to degree 8.
# If -6*n-6+8+1>=0 the series is of importance for the coefficients.

if -6*n-6+8+1<0 then
  su00:=0;
else
  seriessu00:=series(s2,t,-6*n-6+8+1);
  seriessu00:=series(seriessu00^(12),t,-6*n-6+8+1);
  seriessu00:=convert(seriessu00,polynom);
  seriessu00:=collect(simplify(seriessu00),t);
  su00:=collect(t^(6*n+6)*seriessu00,t); 
fi;
 
# Extract the expressions for a9,a6,a3,a0 and the coefficients c2,c4,c6,c8 
# (belonging to t^2, t^4, t^6 and t^8, respectively).


if n=1/6 then
  a9:=-coeff(su12,t,1); 
  a9:=simplify(a9);
  a6:=-coeff(su12,t,3)-a9*coeff(su09,t,3);
  a6:=simplify(a6);
  a3:=-coeff(su12,t,5)-a9*coeff(su09,t,5)-a6*coeff(su06,t,5); 
  a3:=simplify(a3);
  a0:=-coeff(su12,t,7)-a9*coeff(su09,t,7)-a6*coeff(su06,t,7)-a3*coeff(su03,t,7);
  a0:=simplify(a0); 
  c2:=coeff(su12,t,2)+a9*coeff(su09,t,2);
  c2:=simplify(c2);
  c4:=coeff(su12,t,4)+a9*coeff(su09,t,4)+a6*coeff(su06,t,4);
  c4:=simplify(c4);
  c6:=coeff(su12,t,6)+a9*coeff(su09,t,6)+a6*coeff(su06,t,6)+a3*coeff(su03,t,6);
  c6:=simplify(c6);
  c8:=coeff(su12,t,8)+a9*coeff(su09,t,8)+a6*coeff(su06,t,8)+a3*coeff(su03,t,8)
      +a0*coeff(su00,t,8);
  c8:=simplify(c8);
  listofa:=[a9,a6,a3,a0];
  C:=[c2,c4,c6,c8];
else 
  a6:=-coeff(su12,t,3)-a9*coeff(su09,t,3);
  # a6:=collect(a6,a9);
  c1:=coeff(su12,t,1)+a9*coeff(su09,t,1);
  c1:=simplify(c1);
  c2:=coeff(su12,t,2)+a9*coeff(su09,t,2);
  c2:=simplify(c2);
  c4:=coeff(su12,t,4)+a9*coeff(su09,t,4)+a6*coeff(su06,t,4);
  c4:=simplify(c4);
  c5:=coeff(su12,t,5)+a9*coeff(su09,t,5)+a6*coeff(su06,t,5);
  c5:=simplify(c5);
  c6:=coeff(su12,t,6)+a9*coeff(su09,t,6)+a6*coeff(su06,t,6);
  c6:=simplify(c6);
  
  if n=5/6 then 
    a3:=-coeff(su12,t,7)-a9*coeff(su09,t,7)-a6*coeff(su06,t,7);
    a3:=simplify(a3);
    # a3:=collect(a3,a9);
   c8:=coeff(su12,t,8)+a9*coeff(su09,t,8)+a6*coeff(su06,t,8)+a3*coeff(su03,t,8);
    c8:=simplify(c8);
    C:=[c1,c2,c4,c5,c6,c8]; 
  else
    c7:=coeff(su12,t,7)+a9*coeff(su09,t,7)+a6*coeff(su06,t,7);
    c7:=simplify(c7);
    C:=[c1,c2,c4,c5,c6,c7];
  fi;
  C:=subs(a9=inputa9,C);
  listofa:=subs(a9=inputa9,[a9,a6]);
fi;

RETURN(C,listofa);

end:



##########################



eliminateunknown:=proc(Z,E)
     # input: a list E of polynomials in K[Z], where  K is a ring/field 
     #        (we use Q[g2,g3]).
     # Output: E if nops(E) <= 1, 
     #         A list L of polynomials in K, obtained from taking
     #         resultants of two consequtive polynomials of E.
local localE,L,d,i:

localE:=E; 
L:=[];

if nops(localE)<=1 then
  L:=localE;
else
  for i from 1 to floor(nops(localE)/2) do 
    d:=simplify(resultant(localE[2*i-1],localE[2*i],Z));
    L:=[op(L),d];
  od;
fi;

RETURN(L)

end:



##########################



squarefreeeqnsG:=proc(listofpols,X,Y) # reduces the non-zero polynomials in 
    # the given list such that they are square-free.
    # Output: L
    # where L is the list of non-zero square-free polynomials.
   
local locallistpols,nopslist,listsubseqn,subseqn,numbsubseqn,i,factorssubseqn:

locallistpols:=listofpols;
nopslist:=nops(listofpols);

# eliminate all zero-polynomials.

listsubseqn:=[];
for i from 1 to nopslist do 
  subseqn:=(locallistpols[i]);
  if subseqn<>0 then
    listsubseqn:=[op(listsubseqn),subseqn]; 
  fi;
od;

subseqn:='subseqn';  

# make the polynomials square-free.

numbsubseqn:=nops(listsubseqn);
for i from 1 to numbsubseqn do
  factorssubseqn:=factors(listsubseqn[i]);
  factorssubseqn:=convertpolintsquarefree(factorssubseqn); 
  # print(%);
  listsubseqn:=subsop(i=factorssubseqn,listsubseqn); 
od;

RETURN(listsubseqn);

end:



##########################



checkdiscriminant:=proc(listofpols,X,Y)  # for each entry of the list
    # the procedure checks if the discriminant of p(z) is
    # 0 if we substitute its values for g2 and g3 in p(z).
    # If it is, we eliminate this entry from the list.
    # Output: L
    # where L is the list with entry [fX,fY,fYg2].

local localsetsols,nopslocal,polpXY,i,alpha,beta,setsubs1,setsubs2,\
      subsp,discrimsubsp,newsetpols:

localsetsols:=listofpols;
nopslocal:=nops(localsetsols);

polpXY:=4*z^3-X*z-Y;
newsetpols:=[];

for i from 1 to nopslocal do
  if nops(localsetsols[i])>0 then 
    if localsetsols[i][1]<>X or localsetsols[i][2]<>Y then
      if localsetsols[i][1]<>0 and localsetsols[i][3]=0 then 
        alpha:=RootOf(localsetsols[i][1],X);  
        setsubs1:={X=alpha};
        setsubs2:={};
      elif localsetsols[i][1]=0 and localsetsols[i][3]<>0 then
        beta:=RootOf(localsetsols[i][3],Y);
        setsubs1:={Y=beta};
        setsubs2:={};
      elif localsetsols[i][1]<>0 and localsetsols[i][3]<>0 then
        alpha:=RootOf(localsetsols[i][1],X);
        beta:=RootOf(localsetsols[i][3],Y);
        setsubs1:={Y=beta}; 
        setsubs2:={X=alpha};
      fi;

      subsp:=subs(setsubs1,polpXY);
      subsp:=subs(setsubs2,subsp);   
      subsp:=evala(Factor(subsp));  
      discrimsubsp:=evala(discrim(subsp,z));  

      if discrimsubsp<>0 then
        newsetpols :=[op(newsetpols),localsetsols[i]];
      fi;
    fi;
  fi;
od;

RETURN(newsetpols);
   
end;



##########################



invariantsG:=proc(M,n,B,g2,g3,fX,fY,fYg2,beta)  # for the given data
    # this procedure determines beta and gamma
    # such that Jm(y1,y2) is a polynomial in 1/t.
    # Output: beta, gamma, Jm(y1,y2)  if they exist,
    #         {} otherwise.

local alpha,lambda,setsubs1,setsubs2,revset,revsetg2g3,subsbeta,\
      u,locals1,locals2,m,dgamma,dm,dsu,dseries,orders1,orders2,k,lowestseriess1,\
      lowestseriess1s2,su,sv,dsv,sndseriess1,sndseriess2,sndseriess1s2,\
      subsgamma,dsw,highestseriess2,highestseriess1s2,sw,invarpol,subsinvar,\
      returnbeta,returngamma:

# make the set of substitutions for g2 and g3.

if fX<>X or fY<>Y then
  if fX<>0 and fYg2=0 then
    alpha:=RootOf(fX,X);
    lambda:=g3;
    setsubs1:={X=alpha};
    setsubs2:={};
    revset:={alpha=X};
    revsetg2g3:={alpha=g2,Y=g3};
  elif fX=0 and fYg2<>0 then 
    alpha:=g2;
    lambda:=RootOf(fY,Y);
    setsubs1:={Y=lambda};
    setsubs2:={};
    revset:={lambda=Y};
    revsetg2g3:={lambda=g3,X=g2};
  elif fX<>0 and fYg2<>0 then
    alpha:=RootOf(fX,X);
    lambda:=RootOf(fYg2,Y);
    setsubs1:={Y=lambda}; 
    setsubs2:={X=alpha,g2=alpha};
    revset:={lambda=Y,alpha=X};
    revsetg2g3:={alpha=g2,lambda=g3};
  fi; 
fi;

# determine beta for the given g2 and g3.

subsbeta:=subs({g2=X,g3=Y},beta); 
subsbeta:=subs(setsubs1,subsbeta);
subsbeta:=subs(setsubs2,subsbeta); 

if M=G12 then
  m:=6;
elif M=G13 then
  m:=8;
else
  m:=12;
fi;

dgamma:=(m-2)*n/2+(m-1)/2; 
dm:=n*m/2-n-1/2;
dsu:=-n*m/2+n+1/2;

dseries:=dm+max(dgamma,3*m/2 +2);

# determine the series s1 and s2.

orders1:=ceil(dseries);   
orders2:=ceil(dseries);         

# locals1,locals2:=seriess12G(M,n,B,X,Y,orderofs1=orders1,orderofs2=orders2)

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders1 do
  u(k):=(B*u(k-1)+X*(k-2-n/2)*(k-3/2-n/2)*u(k-2)+\
           Y*(k-3-n/2)*(k-2-n/2)*u(k-3));
  u(k):=subs(setsubs1,u(k));
  u(k):=subs(setsubs2,u(k));
  u(k):=evala(u(k)/(4*k*(k-n-1/2)) );
od;
locals1:=sum('u(k)*t^(k)','k'=0..orders1);  
u:='u';

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders2 do
  u(k):=(B*u(k-1)+X*(k-3/2+n/2)*(k-1+n/2)*u(k-2)+\
           Y*(k-3/2+n/2)*(k-5/2+n/2)*u(k-3));
  u(k):=subs(setsubs1,u(k));
  u(k):=subs(setsubs2,u(k));
  u(k):=evala(u(k)/(4*k*(k+n+1/2)) );
od;
locals2:=sum('u(k)*t^(k)','k'=0..orders2); 
u:='u';

# calculate su:= y1^(m-1)*y2(t) up to degree dsu+orders1+1 for the given g2,g3.

lowestseriess1:=series(locals1,t,orders1+1);
lowestseriess1:=evala(series(lowestseriess1^(m-1),t,orders1+1));
lowestseriess1s2:=evala(series(lowestseriess1*locals2,t,orders1+1));
lowestseriess1s2:=convert(lowestseriess1s2,polynom);
lowestseriess1s2:=collect((lowestseriess1s2),t);
su:=collect(t^dsu*lowestseriess1s2,t); 
# su:=collect(t^dsu*lowestseriess1s2,t);

# calculate sv:= y1^m/2*y2^m/2(t) up to degree orderss1+1 for the given g2,g3.

sv:=0;
if M<>G12 then   
  dsv:=m/4;
  sndseriess1:=series(locals1,t,orders1-dsv+1);
  sndseriess1:=evala(series(sndseriess1^(m/2),t,orders1-dsv+1));
  sndseriess2:=series(locals2,t,orders2-dsv+1);
  sndseriess2:=evala(series(sndseriess2^(m/2),t,orders2-dsv+1));
  sndseriess1s2:=evala(series(sndseriess1*sndseriess2,t,orders1-dsv+1));
  sndseriess1s2:=convert(sndseriess1s2,polynom);
  sndseriess1s2:=collect((sndseriess1s2),t);
  sv:=collect(t^dsv*sndseriess1s2,t); 
  # sv:=collect(t^dsv*sndseriess1s2,t);
fi;

# determine gamma for the given g2 and g3.

subsgamma:=-coeff(su,t,dgamma)-subsbeta*coeff(sv,t,dgamma);
subsgamma:=evala(subsgamma);  
# print(subsbeta,subsgamma);

# calculate sw:= y1*y2^(m-1)(t) up to degree orderss2+1 for the given g2,g3.

dsw:=-n/2+(n+1)*(m-1)/2;
highestseriess2:=series(locals2,t,orders2-dsw+1);
highestseriess2:=evala(series(highestseriess2^(m-1),t,orders2-dsw+1));
highestseriess1s2:=evala(series(locals1*highestseriess2,t,orders2-dsw+1));
highestseriess1s2:=convert(highestseriess1s2,polynom);
highestseriess1s2:=collect((highestseriess1s2),t);
sw:=collect(t^dsw*highestseriess1s2,t); 
#sw:=collect(t^dsw*highestseriess1s2,t);

# determine the invariant function su+beta*sv+gamma*sw.

invarpol:=sort(su+subsbeta*sv+subsgamma*sw,t);
invarpol:=evala(series(invarpol,t,3*m/2+2)); 
invarpol:=convert(invarpol,polynom); 
invarpol:=subs(revsetg2g3,invarpol); 
# print(%);

# check if the function is a polynomial in 1/t.

subsinvar:=collect(subs(t=1/z,invarpol),z);  
#HIER MOET EEN BETERE CHECK!!!!!!!!!!!!!!!!!!
if type(subsinvar,polynom(anything,z))=true and degree(subsinvar,z)<=3*m/2+1 then
  if subsbeta=0 then
    returnbeta:=0
  else
    returnbeta:=subs(revsetg2g3,subsbeta);
  fi;
  if subsgamma=0 then
    returngamma:=0
  else
    returngamma:=subs(revsetg2g3,subsgamma);
  fi;
  RETURN(returnbeta,returngamma,invarpol);
else 
  RETURN({});
fi;

end:



##########################



invariantsG22:=proc(n,B,g2,g3,fX,fY,fYg2,a9,a6)  # for the given data
    # this procedure determines a9,a6,a3 and a0
    # such that Jm(y1,y2) is a polynomial in 1/t.
    # Output: beta, gamma, Jm(y1,y2)  if they exist,
    #         {} otherwise.

local alpha,lambda,setsubs1,setsubs2,revset,revsetg2g3,\
      orders1,orders2,k,s1,u,s2,subsdegree,seriess1,seriess2,\
      seriess1power3,seriess2power3,seriessu12,\
      su12,seriessu09,su09,seriessu06,su06,\
      su03,seriessu03,su00,seriessu00,\
      subsa9,subsa6,subsa3,subsa0,abstractpol,testpol,invarpol,subsinvar:

# make the set of substitutions for g2 and g3.

if fX<>X or fY<>Y then
  if fX<>0 and fYg2=0 then
    alpha:=RootOf(fX,X);
    lambda:=g3;
    setsubs1:={X=alpha};
    setsubs2:={};
    revset:={alpha=X};
    revsetg2g3:={alpha=g2,Y=g3};
  elif fX=0 and fYg2<>0 then  
    alpha:=g2;
    lambda:=RootOf(fY,Y);
    setsubs1:={Y=lambda};
    setsubs2:={};
    revset:={lambda=Y};
    revsetg2g3:={lambda=g3,X=g2};
  elif fX<>0 and fYg2<>0 then
    alpha:=RootOf(fX,X);
    lambda:=RootOf(fYg2,Y);
    setsubs1:={Y=lambda}; 
    setsubs2:={X=alpha,g2=alpha};
    revset:={lambda=Y,alpha=X};
    revsetg2g3:={alpha=g2,lambda=g3};
  fi; 
fi;

# determine the series s1 and s2.

orders1:=6*n+max(6*n+8,20);   
orders2:=3*n-3/2+max(6*n+8,20);                

# locals1,locals2:=seriess12G(M,n,B,X,Y,orderofs1=orders1,orderofs2=orders2)

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders1 do
  u(k):=(B*u(k-1)+X*(k-2-n/2)*(k-3/2-n/2)*u(k-2)+\
           Y*(k-3-n/2)*(k-2-n/2)*u(k-3));
  u(k):=subs(setsubs1,u(k));
  u(k):=subs(setsubs2,u(k));
  u(k):=evala(u(k)/(4*k*(k-n-1/2)) );
od;
s1:=sum('u(k)*t^(k)','k'=0..orders1); 
u:='u';

u(-2):=0;
u(-1):=0;
u(0):=1;
for k from 1 to orders2 do
  u(k):=(B*u(k-1)+X*(k-3/2+n/2)*(k-1+n/2)*u(k-2)+\
           Y*(k-3/2+n/2)*(k-5/2+n/2)*u(k-3));
  u(k):=subs(setsubs1,u(k));
  u(k):=subs(setsubs2,u(k));
  u(k):=evala(u(k)/(4*k*(k+n+1/2)) );
od;
s2:=sum('u(k)*t^(k)','k'=0..orders2); 
u:='u';

# determine a number related to the order of the expansion of the series.

subsdegree:=3*n+3/2; 

seriess1:=series(s1,t,orders1+1);
seriess2:=series(s2,t,orders2+1); 
seriess1power3:=evala(series(seriess1^3,t,orders1+1));
seriess2power3:=evala(series(seriess2^3,t,orders2+1));

# calculate su12:= y1^12.

# seriessu12:=series(s1,t,orders1+1); 
seriessu12:=evala(series(seriess1power3^4,t,orders1+1));
seriessu12:=convert(seriessu12,polynom);
seriessu12:=collect(simplify(seriessu12),t);
su12:=collect(t^(-6*n)*seriessu12,t); 

# calculate su09:= y1^9y2^3.

# seriessu09:=series(s1,t,orders1-subsdegree+1);
seriessu09:=evala(series(seriess1power3^3,t,orders1-subsdegree+1));
seriessu09:=evala(series(seriessu09*seriess2power3,t,orders1-subsdegree+1));
seriessu09:=convert(seriessu09,polynom);
seriessu09:=collect(simplify(seriessu09),t);
su09:=collect(t^(-3*n+3/2)*seriessu09,t); 
 
# calculate su06:= y1^6y2^6.

# seriessu06:=series(s1,t,orders1-2*subsdegree+1);
seriessu06:=evala(series(seriess1power3^(2),t,orders1-2*subsdegree+1));
seriessu06:=evala(series(seriessu06*seriess2power3^(2),t,orders1-2*subsdegree+1));
seriessu06:=convert(seriessu06,polynom);
seriessu06:=collect(simplify(seriessu06),t);
su06:=collect(t^(3)*seriessu06,t); 
 
# calculate su03:= y1^3y2^9.

if orders1-3*subsdegree+1<0 then
  su03:=0;
else
  # seriessu03:=series(s1,t,orders1-3*subsdegree+1); 
  seriessu03:=evala(series(seriess1power3,t,orders1-3*subsdegree+1));
  seriessu03:=series(seriessu03*seriess2power3^(3),t,orders1-3*subsdegree+1);
  seriessu03:=convert(seriessu03,polynom);
  seriessu03:=collect(simplify(seriessu03),t);
  su03:=collect(t^(3*n+9/2)*seriessu03,t); 
fi;

# calculate su00:= y2^12.

if orders1-4*subsdegree+1<0 then
  su00:=0;
else
  # seriessu00:=series(s2,t,orders1-4*subsdegree+1);
  seriessu00:=evala(series(seriess2power3^(4),t,orders1-4*subsdegree+1));
  seriessu00:=convert(seriessu00,polynom);
  seriessu00:=collect(simplify(seriessu00),t);
  su00:=collect(t^(6*n+6)*seriessu00,t); 
fi;

if n=1/6 then
  subsa9:=subs({g2=X,g3=Y},a9);
  subsa9:=subs(setsubs1,subsa9);
  subsa9:=subs(setsubs2,subsa9);
  subsa9:=evala(subsa9);
  subsa6:=subs({g2=X,g3=Y},a6);
  subsa6:=subs(setsubs1,subsa6);
  subsa6:=subs(setsubs2,subsa6);
  subsa6:=evala(subsa6);
  subsa3:=-coeff(su12,t,5)-subsa9*coeff(su09,t,5)-subsa6*coeff(su06,t,5); 
  subsa3:=subs(setsubs1,subsa3);
  subsa3:=subs(setsubs2,subsa3);
  subsa3:=evala(subsa3);   subsa0:=-coeff(su12,t,7)-subsa9*coeff(su09,t,7)-subsa6*coeff(su06,t,7)-subsa3*coeff(su03,t,7);
  subsa0:=subs(setsubs1,subsa0);
  subsa0:=subs(setsubs2,subsa0); 
  subsa0:=evala(subsa0); 
else 
  if coeff(su09,t,1)<>0 then
    subsa9:=-coeff(su12,t,1)/coeff(su09,t,1);
    subsa9:=evala(subsa9); 
  elif coeff(su09,t,2)<>0 then
    subsa9:-coeff(su12,t,2)/coeff(su09,t,2);
    subsa9:=evala(subsa9);
  else
    print(`Can't determine if solution is valid`);
    RETURN([]);
  fi; 

  subsa6:=subs({g2=X,g3=Y},a6);
  subsa6:=subs(a9=subsa9,subsa6);
  subsa6:=subs(setsubs1,subsa6);
  subsa6:=subs(setsubs2,subsa6); 
  subsa6:=evala(subsa6);
  subsa3:=-coeff(su12,t,3*n+9/2)-subsa9*coeff(su09,t,3*n+9/2)-
            subsa6*coeff(su06,t,3*n+9/2);
  subsa3:=subs(setsubs1,subsa3);
  subsa3:=subs(setsubs2,subsa3); 
  subsa3:=evala(subsa3);
  subsa0:=-coeff(su12,t,6*n+6)-subsa9*coeff(su09,t,6*n+6) 
           -subsa6*coeff(su06,t,6*n+6) -subsa3*coeff(su03,t,6*n+6);
  subsa0:=subs(setsubs1,subsa0);
  subsa0:=subs(setsubs2,subsa0);
  subsa0:=evala(subsa0);
fi;

# determine the invariant function J12(X,Y).

abstractpol:=A^(12)+subsa9*A^9*K^3+subsa6*A^6*K^6+subsa3*A^3*K^9+subsa0*K^(12); 
abstractpol:=factor(abstractpol);
abstractpol:=subs(revsetg2g3,abstractpol);
abstractpol:=subs({A=X,K=Y}, abstractpol);

lprint(`Checking if J12(X,Y) is a square....`);

testpol:=subs({X=T^(1/3),Y=1},abstractpol);

if discrim(testpol,T)=0 then
  lprint(`  --No solution`);
  lprint(`  --The invariant polynomial J[12](X,Y) contains a square and is`);
  print(abstractpol);
  RETURN({});
fi; 

# determine the invariant function J12(y1,y2).

invarpol:=sort(su12+subsa9*su09+subsa6*su06+subsa3*su03+subsa0*su00,t);
invarpol:=evala(series(invarpol,t,orders1+1));
invarpol:=convert(invarpol,polynom); 
invarpol:=subs(revsetg2g3,invarpol);
# print(%);

# check if the function is a polynomial in 1/t.

subsinvar:=collect(subs(t=1/z,invarpol),z);  
if type(subsinvar,polynom(anything,z))=true then
  lprint(`  --Solution valid.`);
  RETURN(abstractpol,invarpol);
else 
  RETURN({});
fi;

end:


 
##########################





lameG:=proc(M,n,B)  # This procedure uses the procedure reduceeqns2var.

local g2,g3,lameGinput,m,dgamma,dm,s1,s2,E,beta,listE,listeqns,firstlistpols,\
      secondlistpols,L,i,localbeta,localgamma,Jm:
                                             
#check the input.

lameGinput:=inputG(M,n,B);

# define m, dgamma and dm.

m,dgamma,dm:=calculatem(M,n);

# define series around infinity.

s1,s2:=seriess12G(lameGinput,g2,g3); 

# get the first polynomial coefficients and store them in the list E. 
# The entries are polynomials in Q[g2,g3,beta]

E,beta:=listcoefsG(lameGinput,g2,g3,s1,s2); 

# simplify the polynomials in E by deleting 0-polynomials and making polynomials
# square-free.

listE:=squarefreeeqnsG(E,g2,g3);

# solve the system of polynomials of listE.

lprint(`We determine the list of polynomial equations g2 and g3 must satisfy`);
listeqns:=subs({g2=X,g3=Y},listE); 
firstlistpols:=reducebivarpols(listeqns,g2,g3,X,Y); 

# Check if the arrays of fristlistpols does not satisfy discrim(p)=0.

secondlistpols:=checkdiscriminant(firstlistpols,X,Y);

# test if the polynomials give an algebraic Lame equation.


lprint(`The following list of candidate solutions\
 [pol(g2)/Q, pol(g3)/Q, pol(g3)/Q(g2)] as entries`);
lprint(`needs to be checked as being solutions of the Lame' equation`);
print(secondlistpols);


lprint(`Checking the candidate solutions....`);
lprint(`  --This may take a while.`);

L:=[];
for i from 1 to nops(secondlistpols) do
  lprint(`Checking candidate no.`, i, `of`, nops(secondlistpols));
  localbeta,localgamma,Jm:=invariantsG(lameGinput,g2,g3,op(secondlistpols[i]),beta);
  if Jm<>{} then
    L:=[ op(L) , [lameGinput,op(secondlistpols[i]),localbeta,localgamma,Jm] ];
  fi;
od;

print();
print(` ******************************************************************`);
print(`The list of all algebraic Lame' equations with M, n, B =`, lameGinput);
print(`is given by the following list with entries`); 
print(`[M, n, B, pol(g2)/Q, pol(g3)/Q, pol(g3)/Q(g2), beta, gamma, Jm(y1,y2)(t)].`);

RETURN(L);

end;




##########################





lameG22:=proc(n,B)  # This procedure uses the procedure reduceeqns2var.

local g2,g3,lameG22input,s1,s2,E,lista,eliminatedE,listE,listeqns,firstlistpols,\
      secondlistpols,L,i,J12data:
                                             
#check the input.

lameG22input:=inputG22(n,B);

# define series around infinity.

s1,s2:=seriess12G(G22,lameG22input,g2,g3); 

# get the first polynomial coefficients and store them in the list E. 
# The entries are polynomials in Q[g2,g3] if n<=1/6,
# and in Q[g2,g3,a9] otherwise

E,lista:=listcoefsG22(lameG22input,g2,g3,s1,s2,a9); 

# get equations in Q[g2,g3] by using resultants.

if n>1/6 then
  eliminatedE:=eliminateunknown(a9,E);
else 
  eliminatedE:=E;
fi;

# simplify the polynomials in eliminatedE by deleting 0-polynomials and making 
# polynomials square-free.

listE:=squarefreeeqnsG(eliminatedE,g2,g3);

# solve the system of polynomials of listE.

lprint(`We determine the list of polynomial equations g2 and g3 must satisfy`);
listeqns:=subs({g2=X,g3=Y},listE); 
firstlistpols:=reducebivarpols(listeqns,g2,g3,X,Y); 

# Check if the arrays of fristlistpols does not satisfy discrim(p)=0.

secondlistpols:=checkdiscriminant(firstlistpols,X,Y); 

# test if the polynomials give an algebraic Lame equation.

lprint(`The following list of candidate solutions\
 [pol(g2)/Q, pol(g3)/Q, pol(g3)/Q(g2)] as entries`);
lprint(`needs to be checked as being solutions of the Lame' equation`);
print(secondlistpols);


lprint(`Checking the candidate solutions....`);
lprint(`  --This may take a while.`);

L:=[];
for i from 1 to nops(secondlistpols) do
  lprint(`Checking candidate no.`, i, `of`, nops(secondlistpols));
  J12data:=invariantsG22(lameG22input,g2,g3,op(secondlistpols[i]),op(lista));
  if J12data<>{} and J12data<>[]then
    L:=[ op(L) , [G22,lameG22input,op(secondlistpols[i]),J12data] ];
  fi;
od;

print();
print(` ******************************************************************`);
print(`The list of all algebraic Lame' equations with M, n, B =`, G22, lameG22input);
print(`is given by the following list with entries`); 
print(`[G22, n, B, pol(g2)/Q, pol(g3)/Q, pol(g3)/Q(g2), J[12](X,Y), J[12](y1,y2)(t)].`);

RETURN(L);

end;




##########################
##########################
##########################
##########################
##########################
##########################




